#ifndef CHANNELMAPSERVICE__H
#define CHANNELMAPSERVICE__H
////////////////////////////////////////////////////////////////////////
/// \file ChannelMap_service.cc
///
///
/// \author Andrey Sheshukov
////////////////////////////////////////////////////////////////////////

#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Run.h"
#include "art/Framework/Principal/SubRun.h"
#include "art/Utilities/InputTag.h"
#include "fhiclcpp/ParameterSet.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Services/Registry/ActivityRegistry.h"
#include "art/Framework/Services/Registry/ServiceMacros.h"
#include "messagefacility/MessageLogger/MessageLogger.h"

#include "OnlineCalibration/SharedNoiseMap.h"

namespace novaddt{
    class ChannelMapService;
    class MapSource;
}

class novaddt::MapSource{
    public:
        MapSource(fhicl::ParameterSet const& pset);      
    public:
        std::string type;
        std::string name;
        int count;
};


//A service for access to shared noise hit map object

class novaddt::ChannelMapService{
    public:
        ChannelMapService(fhicl::ParameterSet const& pset, art::ActivityRegistry& reg);
        //get the scoped lock for reading/writing buffer.
        novaddt::calib::sharableLock GetLock();
        //get map for reading
        const novaddt::calib::FDHitMap& GetRMap() const;
        //get map for writing
        novaddt::calib::FDHitMap& GetWMap();
        //append counter - until it reaches counterMax. Then buffers will rotate
        void addCounter(int n=1){fSharedMapContainer.getData().addCounter(n);}
        size_t getCounter(){return fSharedMapContainer.getData().getCounter();}
        size_t getCounterMax(){return fSharedMapContainer.getData().getCounterMax();}
    private:
        void preBeginRun(const art::Run& run);
        void postEndRun(const art::Run& run);
        void postProcessEvent(const art::Event& event);
        void fillMapFromEvent(const art::Event& event, const art::InputTag& product);
        void readMapFromFile(const std::string& fname);
        void saveMapToFile(const std::string& fname);
        
       novaddt::MapSource fMapSource; //< configuration for map source
       std::string fSegName;
       std::string fMapName;
       novaddt::calib::SharedFDHitMap fSharedMapContainer;
};

DECLARE_ART_SERVICE(novaddt::ChannelMapService,LEGACY);
#endif
