////////////////////////////////////////////////////////////////////////
// Class:       NoiseHitFinder
// Module Type: producer
// File:        NoiseHitFinder_module.cc
//
// Generated at Thu Apr 14 03:19:48 2016 by Andrey Sheshukov using artmod
// from cetpkgsupport v1_08_07.
////////////////////////////////////////////////////////////////////////

#include "art/Framework/Core/EDProducer.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Run.h"
#include "art/Framework/Principal/SubRun.h"
#include "art/Utilities/InputTag.h"
#include "fhiclcpp/ParameterSet.h"
#include "messagefacility/MessageLogger/MessageLogger.h"

#include "DDTBaseDataProducts/BaseProducts.h"
#include "DDTBaseDataProducts/HitList.h"
#include "DDTBaseDataProducts/DAQHit.h"
#include "DDTBaseDataProducts/HitSet.h"

#include "OnlineCalibration/ChannelMapService.h"

#include <memory>

namespace novaddt {
  class NoiseHitFinder;
}

class novaddt::NoiseHitFinder : public art::EDProducer {
public:
  explicit NoiseHitFinder(fhicl::ParameterSet const & p);
  // The destructor generated by the compiler is fine for classes
  // without bare pointers or other resource use.

  // Plugins should not be copied or assigned.
  NoiseHitFinder(NoiseHitFinder const &) = delete;
  NoiseHitFinder(NoiseHitFinder &&) = delete;
  NoiseHitFinder & operator = (NoiseHitFinder const &) = delete;
  NoiseHitFinder & operator = (NoiseHitFinder &&) = delete;

  // Required functions.
  void produce(art::Event & e) override;


private:

  // Declare member data here.
  std::string _InputTag,_OutputTag;
  uint32_t _NoiseMin, _NoiseMax;
};


novaddt::NoiseHitFinder::NoiseHitFinder(fhicl::ParameterSet const & p):
    _InputTag     (p.get< std::string >("inputTag")),
    _OutputTag    (p.get< std::string >("outputTag")),
    _NoiseMin(p.get< uint32_t >("Noise.min")),
    _NoiseMax(p.get< uint32_t >("Noise.max"))
{

    produces<HitList>(_OutputTag);
   //output configuration
    mf::LogInfo("NoiseHitFinder")<<"config:"
        <<"\n\t inputTag="<<_InputTag
        <<"\n\t outputTag="<<_OutputTag
        <<"\n\t Noise limits: ["<<_NoiseMin<<" - "<<_NoiseMax<<"]";
}

void novaddt::NoiseHitFinder::produce(art::Event & event)
{
    //read the input hit list
    auto inputHits=event.getValidHandle<novaddt::HitList>(_InputTag);
    //prepare the output hit list
    std::unique_ptr<novaddt::HitList> outputHits(new novaddt::HitList);
    ///use the shared noise map manager
    art::ServiceHandle<novaddt::ChannelMapService> cms;
    //lock before reading
    auto lock = cms->GetLock();
    auto &NoiseMap = cms->GetRMap();
    for(auto const& h: *inputHits){
        auto noise=NoiseMap[h];
        if((noise<=_NoiseMax)&&(noise>=_NoiseMin))
            outputHits->emplace_back(h);
    }
    mf::LogDebug("NoiseHitFinder")<<" find hits"
        <<"\n\tinputHits:  "<< inputHits->size()
        <<"\n\toutputHits: "<<outputHits->size();
    event.put(std::move(outputHits),_OutputTag);

}

DEFINE_ART_MODULE(novaddt::NoiseHitFinder)
